context("CSTools::CST_ProxiesAttractor tests")

##############################################

# dat1
set.seed(1)
data <- rnorm(2 * 10 * 4 * 5 * 6 * 7)
dim(data) <- c(dataset = 2, member = 10,
                sdate = 4, ftime = 5, lat = 6, lon = 7)
lon <- seq(0, 12, 2)
lat <- seq(10, 15, 1)
coords <- list(lon = lon, lat = lat)
exp1 <- list(data = data, coords = coords)
attr(exp1, "class") <- "s2dv_cube"

##############################################
test_that("1. Input checks", {
  # 's2dv_cube'
  expect_error(
    CST_ProxiesAttractor(rnorm(2 * 15 * 4 * 5 * 6 * 7)),
    paste0("Parameter 'data' must be of the class 's2dv_cube'.")
  )
  # quanti
  expect_error(
    CST_ProxiesAttractor(data = exp1, quanti = NULL),
    paste0("Parameter 'quanti' cannot be NULL.")
  )
  # spatial_dims
  expect_error(
    ProxiesAttractor(data = data, quanti = 0.6, spatial_dims = 'lons'),
    paste0("Parameter 'spatial_dims' is not found in 'data' dimension names. ", 
           "Please check the dimension names of 'data' or set it to NULL.")
  )
  # time_dims
  expect_error(
    ProxiesAttractor(data = data, quanti = 0.6, time_dims = c('t')),
    paste0("Parameter 'time_dims' is not found in 'data' dimension names. ", 
           "Please check the dimension names of 'data'.")
  )
})
##############################################
test_that("2. Output checks", {
  expect_equal(
    dim(ProxiesAttractor(data = data, quanti = 0.6)$dim),
    c(sdate = 4, dataset = 2, member = 10, ftime = 5)
  )
  expect_equal(
    dim(ProxiesAttractor(data = data, quanti = 0.6)$theta),
    c(sdate = 4, dataset = 2, member = 10, ftime = 5)
  )
  expect_equal(
    as.vector(ProxiesAttractor(data = data, quanti = 0.6)$dim[, , 1, 1])[1:5], 
    c(7.197594, 8.781479, 7.343547, 9.659450, 11.198145), 
    tolerance = 0.0001
  )
})

##############################################

