#*****************************************************************************
# Hands-On exercise: easyNCDF
# An-Chi - 30/05/2022
#*****************************************************************************

# 0. Libraries used
library(easyNCDF)
library(abind)

# 1. Let’s practice with  ERA5 sea level pressure (psl) monthly mean data ( /esarchive/recon/ecmwf/era5/monthly_mean/psl_f1h/). Explore the file with easyNCDF functions like NcReadDims(), NcReadVarNames().
file_path= '/esarchive/recon/ecmwf/era5/monthly_mean/psl_f1h/psl_201911.nc'
fnc <- NcOpen(file_path)
fnc_dims <- NcReadDims(fnc)
var_names <- NcReadVarNames(fnc)
fnc_dims
var_names

# 2. Read November 2019 psl variable with NcToArray. Play a bit with all the function parameters.
a_psl_11 <- NcToArray(fnc,var='psl') # only var psl
a_all_11 <- NcToArray(fnc,var=var_names) # all vars
a_lat_lon_11 <- NcToArray(fnc, list(var=c('lat','lon')))
#a_from_file_two_vars <- NcToArray(fnc, list(var=list(c('lon'),c('lat'))))

# 3. Read the same file as above but load longitude and latitude variables (1) with psl as one array (2) as separate 3 arrays.
a_lon_11 <- NcToArray(fnc,var = "lon")
a_lat_11 <- NcToArray(fnc,var = "lat")

NcClose(fnc)

# 4. Check the dimensions of the above arrays. Check the range and order of latitude and longitude
dim(a_lat_11)
dim(a_lon_11)
dim(a_psl_11)
dim(a_all_11)

a_all_11[1,1,1,1,1]
summary(a_all_11[,,1,,])
summary(a_psl_11)
range(a_lat_11)
range(a_lon_11)
range(a_psl_11)

as.vector(a_lat_11) # only the data
a_lat_11[1]

# 5. Read the same psl data but all the months in 2019. Combine the arrays into a single array with function abind::abind.
path_2019 <- "/esarchive/recon/ecmwf/era5/monthly_mean/psl_f1h/"
labs <- c(paste("psl_20190",1:9,sep="",".nc"),paste("psl_2019",10:12,sep="",".nc"))

# initialization of the array (for the first month)
fnc_01 <- NcOpen(paste(path_2019, labs[1],sep=""))
a_psl_01 <- NcToArray(fnc_01,var='psl')
var_names_01 <- NcReadVarNames(fnc_01)
dim_names_01 <- NcReadDims(fnc_01)
dim(a_psl_01)

# for loop to read 2019 data files
a_psl_all_o <- array(dim=c(1,1280, 640, 12)) # another way without bind

for (lab in labs[2:12]) {fnc_i <- NcOpen(paste(path_2019, lab,sep="")) 
                    a_psl_i <- NcToArray(fnc_i,var='psl')
                    print(lab)
                    print(dim(a_psl_01))
                    a_psl_01 <- abind(a_psl_01, a_psl_i)}
dim(a_psl_01)

# Same dimension names:
names(dim(a_psl_01)) <- names(dim_names_01[2:5])
dim(a_psl_01)
var(a_psl_01)

# 6. Save the array as a netCDF file under your own repository by ArrayToNc(<the_array_object>, '/esarchive/scratch/erifarov/<file_name>.nc')
file_path_save_2019 <- '/esarchive/scratch/erifarov/eva_r_train/ho_easyNCDF/a_psl_2019.nc'
ArrayToNc(list(psl = a_psl_01),file_path_save_2019)

#7. Check the structure of the file with ncdump -h (available after doing "module load netCDF" in a terminal session). What are the variable and dimension names?
##---------------------------------------
#dimensions:
#        bnds = 1 ;
#        lon = 1280 ;
#        lat = 640 ;
#        time = UNLIMITED ; // (12 currently)
#variables:
#        double psl(time, lat, lon, bnds) ;
##---------------------------------------

#8. Keep the dimensions 'lon = 1280', 'lat = 640', and 'time = 1', and assign names to the dimensions of your array and save it again. Check it with ncdump.
# --> Ok keep names. How do I keep the dimensions, do I have to put time from 12 to 1 dimension (1 year)?

#9. Create another sample array with no coinciding dimension names and save both into a NetCDF with a single ArrayToNc call. Check it with ncdump.
a9 <- array(data = 1:27, dim = c(a= 10,b=5,c=2))
dim(a9)

ArrayToNc(list(a_psl_01, a9), '/esarchive/scratch/erifarov/eva_r_train/ho_easyNCDF/ex9.nc')
#--------------------------------------
# dimensions:
#         bnds = 1 ;
#         lon = 1280 ;
#         lat = 640 ;
#         time = UNLIMITED ; // (12 currently)
#         a = 10 ;
#         b = 5 ;
#         c = 2 ;
# variables:
#         double var1_1(time, lat, lon, bnds) ;
#         int var2_1(c, b, a) ;
# data:
#  var1_1 =
#--------------------------------------

#10. Add a dimension 'var' of length = 1 to one of your arrays, and save it to a NetCDF file. Check it with ncdump.

a9_add_var <- array(a9,dim=c(dim(a9), 'var' = 1)) # another way
# dim(a9_add_var) <- c(dim(a9), var = 1)
dim(a9_add_var)
file_path_add_var <- '/esarchive/scratch/erifarov/eva_r_train/ho_easyNCDF/ex9_add_var.nc'
ArrayToNc(list(a9_add_var),file_path_add_var)
# --> I save it and it doesn't appear new var. Should I do it not using the old vector?
# Or with dimension names
# 'var' dimension name will generate multiple variables in the 
# resulting NetCDF file
a <- array(1:27, dim = c(3, 3, 3))
names(dim(a)) <- c('lon', 'lat', 'var')
dim(a)

ArrayToNc(a, '/esarchive/scratch/erifarov/eva_r_train/ho_easyNCDF/tmp.nc')


#11. Like 10. but var of length = 2. Save it to a NetCDF file. Check it with ncdump.
a9_add_var2 <- array(a9,dim=c(dim(a9), 'var' = 2))
dim(a9_add_var2)
dim(a9)

ArrayToNc(a9_add_var2, '/esarchive/scratch/erifarov/eva_r_train/ho_easyNCDF/a9_var2.nc')


#12. Create a third array, which has one or more dimensions with matching dimension names in the first array. Save the new array together with the first one into a NetCDF file. Check it with ncdump.

a12_n <- array(1,dim = c( lon=1280,lat=640))
a12_m <- array(1,dim=c(var = 1,lon = 1280,lat=640,time=1))
dim(a12_n)

dim(a12_m)
ArrayToNc(list(a12_n,a12_m), '/esarchive/scratch/erifarov/eva_r_train/ho_easyNCDF/ex12.nc')

#13. Add a 'time' dimension to the first position of the second array and save it into a NetCDF file. Put the 'time' dimension in the last position of the array and save it into a NetCDF file. One of them should crash.
a13 <- array(1,dim=c(time=1,a=1,b=2,c=2,d=3))

a13 <- array(c(2, a9),dim= c(time=2,dim(a9)))
dim(a13)
a13
ArrayToNc(a13, '/esarchive/scratch/erifarov/eva_r_train/ho_easyNCDF/ex13.nc')
#--------------------- > Error

a13 <- array(1,dim=c(a=1,b=2,c=2,d=3,time=1))
dim(a13)
a13
ArrayToNc(a13, '/esarchive/scratch/erifarov/eva_r_train/ho_easyNCDF/ex13.nc')
# ---------------------- > It doesn't crashes because it is in the last position?

#14. Try to save the 3rd array in a NetCDF file where the variable has the attributes 'hello = 2' and 'bye = "three"'. Check the examples in ?ArrayToNc to discover how that is done.
a14 <- array(1,dim= c(a=1,b=2,c=2,d=3,time=1))
dim(a14)
attr(a14,'variables') <- list(psl = list(hello = '2', bye = 'three'))
ArrayToNc(a14, '/esarchive/scratch/erifarov/eva_r_train/ho_easyNCDF/ex14.nc')

#15. Try to save one of your arrays in a NetCDF file with a global attribute 'global = "i am global"'.
a15 <- array(1, dim = c(a=1,b=2,c=3,d=4,time=1))
dim(a15)
attrs <- list(variables = list(aaa = list(hello = '2', bye = 'three')), global_attrs = list(global_attrs= 'global = "i am global"'))
attributes(a15) <- c(attributes(a15), attrs)

ArrayToNc(a15, '/esarchive/scratch/erifarov/eva_r_train/ho_easyNCDF/ex15.nc')
#-------------------------------------------------------------------------------
