################################################################################
# (1) Example of WF of the CSSuite
# Script that runs the CSSuite

source("modules/Loading/Loading.R")
source("modules/Calibration/Calibration.R")
source("modules/Anomalies/Anomalies.R")
source("modules/Skill/Skill.R")
source("modules/Saving/Saving.R")
source("modules/Visualization/Visualization.R")

recipe <- prepare_outputs(recipe_file = "modules/Loading/testing_recipes/recipe_seasonal-tests.yml")

# Load datasets
data <- load_datasets(recipe = recipe)
# Calibrate datasets
data <- calibrate_datasets(recipe = recipe, data = data)
# Compute skill metrics 
skill_metrics <- compute_skill_metrics(recipe = recipe, data = data)
# Compute percentiles and probability bins
probabilities <- compute_probabilities(recipe = recipe, data = data)
# Export all data to netCDF
save_data(recipe = recipe, data = data, skill_metrics = skill_metrics,
          probabilities = probabilities)
# Plot data
plot_data(recipe = recipe, data = data, skill_metrics = skill_metrics,
          probabilities = probabilities, significance = T)
################################################################################
# (2) Code to compute NAO from Nuria

# Step (0): Set the WD to local
setwd("/esarchive/scratch/erifarov/git_forks/s2s-suite")

# Step (1): Load all the modules
source("modules/Loading/Loading.R")
source("modules/Calibration/Calibration.R")
source("modules/Anomalies/Anomalies.R")
source("modules/Skill/Skill.R")
source("modules/Saving/Saving.R")
source("modules/Visualization/Visualization.R")
source("modules/Indices/Indices.R")

# Step (2): Check the recipe
recipe_file <- "/esarchive/scratch/erifarov/git_forks/s2s-suite/recipes/NAO_recipe.yml"
recipe <- prepare_outputs(recipe_file)

# Step (3): Load datasets
data <- load_datasets(recipe)
data_loaded <- readRDS("/esarchive/scratch/erifarov/rpackages/auto-s2s/Test_NAOmodule.RDS")

gc()

# Step (4): Compute_anomalies
data <- compute_anomalies(recipe, data)

# Step (5): Compute module NAO - Indices
nao_s2dv <- Indices(data, recipe)

################################################################################
