

# ---------------------------------- libraries ---------------------------------

library(shiny)
library(shinyjs)
library(shinyBS)
library(plyr)
library(abind)
library(magrittr)


# ---------------------------------- log file ----------------------------------

# disabled
# file('/esarchive/scratch/erifarov/shiny/v2.0.0/new_shinyApp/log-shiny/dev-server-app.txt', open = 'a')
# sink('/esarchive/scratch/erifarov/shiny/v2.0.0/new_shinyApp/log-shiny/dev-server-app.txt')

# ----------------------------- functions --------------------------------------

func_pathinfo <- function(exp, table = 'ocean', info = 'plot_type') {

  path <- file.path('/esarchive/exp/ecearth', exp, 'plots_s2d', table)
  vars <- list.files(path)

  # check name var
  name_var <- c('heatc0-300m', 'pr', 'psl', 'tas', 'tosmean', 'sie', 'sithick', 'siv')
  qwe <- sapply(vars, function(x) {if (!(any(x %in% name_var))) {x <- 0} else {x <- 1} })
  vars <- vars[which(qwe == 1)]

  allfiles <- list.files(paste0(path, '/', vars))
  allfiles <- stringr::str_sub(allfiles,0,-5)
  split_pieces <- stringr::str_split(allfiles, pattern = '_')

  # check figure name
  fig_types <- c('figIa1', 'figIa2', 'figII', 'figIII', 'figIVa', 'figIVb', 'figV', 'figVI')
  elem1 <- lapply(split_pieces, `[[`, 1)
  elem2 <- lapply(split_pieces, `[[`, 2)
  q1 <- sapply(elem1, function(x) {if (!(x %in% c('map', 'timeseries'))) {x <- 0} else {x <- 1} })
  q2 <- sapply(elem2, function(x) {if (!(x %in% fig_types)) {x <- 0} else {x <- 1} })

  split_pieces <- split_pieces[which(q1 == 1 & q2 == 1)]

  if (info == 'plot_type') {
    info_vector <- sapply(split_pieces, '[[', 2)
  } else if (info == 'var') {
    info_vector <- sapply(split_pieces, '[[', 4)
  }

  return(info_vector)
}

func_plottype <- function(plot_type_full_list, var_full_list, plot_type_name, 
                          selected_var_UI_name, select_all_i, select_module = FALSE) {
  
  ind <- which(plot_type_full_list == plot_type_name)
  var <- unique(var_full_list[ind])
  
  # pre-defined list for UI titles
  if (plot_type_name %in% c('figIa1')) {
    ui_title <- 'Raw timeseries (1st half):'
  } else if (plot_type_name %in% c('figIa2')) {
    ui_title <- 'Raw timeseries (2nd half):'
  } else if (plot_type_name %in% c('figII')) {
    ui_title <- 'Anomaly timeseries (1st FY):'
  } else if (plot_type_name %in% c('figIII')) {
    ui_title <- 'Lead-time dependent climatology:'
  } else if (plot_type_name %in% c('figIVa')) {
    ui_title <- 'ACC of timeseries (vs OBS):'
  } else if (plot_type_name %in% c('figIVb')) {
    ui_title <- 'ACC of timeseries (vs RECONS):'
  } else if (plot_type_name %in% c('figV')) {
    ui_title <- 'Bias maps:'
  } else if (plot_type_name %in% c('figVI')) {
    ui_title <- 'ACC maps:'
  } else {  # leftover
    ui_title <- plot_type_name
  }
  
  if (isTRUE(select_module)) {
    selected_var <- select_all_i
  } else {
    # Decide to select all or NULL
    if (select_all_i %% 2 == 0) {
      selected_var <- NULL
    } else {
      selected_var <- var
    }
  }
  
  if (!identical(var, character(0))) {
      output <- checkboxGroupInput(selected_var_UI_name, ui_title,
                                   choices = var, selected = selected_var)
  } else {
    output <- NULL
  }
  return(list(output = output))
}
#####################
extendedCheckboxGroupInput <- function(..., extensions = list()) {
  cbg <- checkboxGroupInput(...)
  nExtensions <- length(extensions)
  nChoices <- length(cbg$children[[2]]$children[[1]])

  if (nExtensions > 0 && nChoices > 0) {
    lapply(1:min(nExtensions, nChoices), function(i) {
      cbg$children[[2]]$children[[1]][[i]]$children[[2]] <<- extensions[[i]]
    })
  }
  cbg
}

collapsibleCheckboxGroupInput <- function(inputId, label, i, choices = NULL, selected = NULL, width = NULL, 
                                          choiceNames = NULL, choiceValues = NULL, extensions) {
  input <- extendedCheckboxGroupInput(inputId, label, choices = choices, 
                                      selected = selected, width = width,
                                      choiceNames = choiceNames, choiceValues = choiceValues,
                                      extensions = extensions)
  checkboxes <- input[[3]][[2]][[3]][[1]]
  id_btn <- paste0(inputId, "_btn")
  id_div <- paste0(inputId, "_collapsible")
  btn <- actionButton(id_btn, "More...", 
                      icon = icon("collapse-up", lib = "glyphicon"), 
                      class = "btn-primary btn-sm", `data-toggle`="collapse", 
                      `data-target` = paste0("#", id_div), style='padding:2px; font-size:80%')
  collapsible <- div(id = id_div, class = "collapse")
  collapsible$children <- checkboxes[(i+1):length(checkboxes)]
  children <- c(checkboxes[1:i], list(btn), list(collapsible))
  input[[3]][[2]][[3]][[1]] <- children
  script <- sprintf('$(document).ready(function(){
                     $("#%s_collapsible").on("hide.bs.collapse", function(){
                     $("#%s_btn").html("<span class=\\\"glyphicon glyphicon-collapse-down\\\"></span> More...");
                     });
                     $("#%s_collapsible").on("show.bs.collapse", function(){
                     $("#%s_btn").html("<span class=\\\"glyphicon glyphicon-collapse-up\\\"></span> Less...");
                     });
                     });', inputId, inputId, inputId, inputId)
  tagList(input, tags$script(HTML(script)))
}


func_plottype2 <- function(plot_type_full_list, var_full_list, plot_type_name, 
                          selected_var_UI_name, pB_id, select_all_i, select_module = FALSE) {
  
  ind <- which(plot_type_full_list == plot_type_name)
  var <- unique(var_full_list[ind])
  
  # pre-defined list for UI titles
  if (plot_type_name %in% c('figIa1')) {
    ui_title <- 'Raw timeseries (1st half):'
  } else if (plot_type_name %in% c('figIa2')) {
    ui_title <- 'Raw timeseries (2nd half):'
  } else if (plot_type_name %in% c('figII')) {
    ui_title <- ' Anomaly timeseries (1st FY):'
  } else if (plot_type_name %in% c('figIII')) {
    ui_title <- 'Lead-time dependent climatology:'
  } else if (plot_type_name %in% c('figIVa')) {
    ui_title <- 'ACC of timeseries (vs OBS):'
  } else if (plot_type_name %in% c('figIVb')) {
    ui_title <- 'ACC of timeseries (vs RECONS):'
  } else if (plot_type_name %in% c('figV')) {
    ui_title <- 'Bias maps:'
  } else if (plot_type_name %in% c('figVI')) {
    ui_title <- 'ACC maps:'
  } else {  # leftover
    ui_title <- plot_type_name
  }

  # Tooltip for var
  long_name_var <- list(
    tas = "Surface Air Temperature",
    psl = "Sea Level Pressure",
    pr = "Precipitation",
    tosmean = "Sea Surface Temperature",
    "heatc0-300m" = "Ocean Heat Content in Upper 300m",
    sie = "Sea Ice Extent",
    siv = "Sea Ice Volume",
    sic = "Sea Ice Concentration",
    sithick = "Sea Ice Thickness"
  )
  
  ind_list <- sapply(lapply(var, '==', names(long_name_var)), which)
  tooltip_list <- vector('list', length = length(ind_list))

  if (isTRUE(select_module)) {
    selected_var <- select_all_i
  } else {
    # Decide to select all or NULL
    if (select_all_i %% 2 == 0) {
      selected_var <- NULL
    } else {
      selected_var <- var
    }
  }
  for (kk in 1:length(tooltip_list)) {
    if (!identical(ind_list[[kk]], integer(0))) {
      tooltip_list[[kk]] <- tipify(bsButton(paste0('pB', pB_id), "",
                                            icon = icon("info-circle", class = NULL, lib = "font-awesome"),
                                            style = "inverse", size = "extra-small"),
                                   long_name_var[[ind_list[[kk]]]])
    } else {
      tooltip_list[[kk]] <- tipify(bsButton(paste0('pB', pB_id), "",
                                            icon = icon("info-circle", class = NULL, lib = "font-awesome"), 
                                            style = "inverse", size = "extra-small"),
                                   var[kk])
    }
    pB_id <- pB_id + 1
  }

  if (!identical(var, character(0))) {
      if (length(var) <= 4) {
        output <- extendedCheckboxGroupInput(selected_var_UI_name, ui_title,
                                             choices = var,
                                             selected = selected_var,
                                             extensions = tooltip_list)
      } else {
        output <- collapsibleCheckboxGroupInput(selected_var_UI_name, ui_title, 
                                                i = 2, choices = var,
                                                selected = selected_var,
                                                extensions = tooltip_list)
      }
  } else {   # should not happen
    output <- NULL
  }
  return(list(output = output, pB_id = pB_id))
}

# ---------------------------------- ui module ---------------------------------

MultipleExpUI <- function(id) {
  ns <- NS(id)
  tagList(
    uiOutput(ns("exp")),
    uiOutput(ns('table_1')),
    actionButton(ns("nextPlot"),"Show figures"),
    actionButton(ns("unselect_all"), "Unselect all"),
    actionButton(ns("select_all"), "Select all" ),
    uiOutput(ns("figIa1_var_table")),
    uiOutput(ns("figIa2_var_table")),
    uiOutput(ns("figII_var_table")),
    uiOutput(ns("figIII_var_table")),
    uiOutput(ns("figIVa_var_table")),
    uiOutput(ns("figIVb_var_table")),
    uiOutput(ns("figV_var_table")),
    uiOutput(ns("figVI_var_table")),
    uiOutput(ns('plots'))
  )
}

# -------------------------------- server module -------------------------------

MultipleExpServer <- function(input, output, session, figure_size, pB_id) {

  ns <- session$ns
  values <- reactiveValues()
  if (!exists('values$pB_id')) values$pB_id <- pB_id

  observeEvent(values, {
        values$pB_id <- values$pB_id()
    }, once = TRUE) 
  observe({
    # check directory /plots_s2d
    asd <- list.files(file.path('/esarchive/exp/ecearth/'))
    asd_fullpath <- lapply(asd, function(x) paste0('/esarchive/exp/ecearth/', x, '/'))
    qwe <- lapply(asd_fullpath, function(x) list.files(x))
    qwe <- sapply(qwe, function(x) {if (!('plots_s2d' %in% x)) {x <- 0} else {x <- 1} })
    asd <- asd[which(qwe == 1)]

    # check tables
    fullpathexp <- lapply(asd, function(x) paste0('/esarchive/exp/ecearth/', x, '/plots_s2d/'))
    qwexp <- lapply(fullpathexp, function(x) list.files(x))
    qwexp <- sapply(qwexp, function(x) {if (!(all(x %in% c('atmos', 'ocean', 'seaice')))) {x <- 0} else {x <- 1} })
    asd <- asd[which(qwexp == 1)]

    output$exp <- renderUI({
      selectizeInput(ns("exp_1"), "Experiment ID:", choices = asd, selected = "a3w5")
    }) #renderUI, output$exp_1
  })

  tables <- eventReactive({
    input$exp_1
    1}, {
      req(!is.null(input$exp_1))
      opt_table <- list.files(file.path('/esarchive/exp/ecearth',
                                        input$exp_1, 'plots_s2d'))
      return(opt_table)
    })
  
  output$table_1 <- renderUI({
    selectizeInput(ns("tab_1"), "Tables:", choices = tables(), selected = tables()[1])
  }) #renderUI, output$tab_1
  
  observeEvent({
    input$exp_1
    input$tab_1
    -99999}, {
      req(!is.null(input$tab_1))
      predefined_plot_type <- c('figIa1', 'figIa2', 'figII', 'figIII',
                                'figIVa', 'figIVb', 'figV', 'figVI')
      plot_type_list_table <- func_pathinfo(input$exp_1, input$tab_1, 'plot_type')
      var_list_table <- func_pathinfo(input$exp_1, input$tab_1, 'var')
      for (plot_type_name in unique(plot_type_list_table)) {
        local({
          selected_var_UI_name <- ns(paste0(plot_type_name, '_','var_', 'table'))
          var <- unique(var_list_table[which(plot_type_list_table == plot_type_name)])       
          checkbox <- func_plottype2(plot_type_full_list = plot_type_list_table, var_full_list = var_list_table, 
                                     plot_type_name = plot_type_name, selected_var_UI_name = selected_var_UI_name, 
                                     pB_id = values$pB_id, select_all_i = var[1], select_module = TRUE)
          selected_var_UI_name <- paste0(plot_type_name, '_','var_', 'table')
          output[[selected_var_UI_name]] <- renderUI(checkbox[[1]]) 
          values$pB_id <- checkbox[[2]]
        })
      }
      
      if (!all(predefined_plot_type %in% plot_type_list_table)) {
        plot_type_name_noexist <- predefined_plot_type[which(!(predefined_plot_type %in% plot_type_list_table))]
        for (plot_noexist in plot_type_name_noexist) {
          local({
            selected_var_UI_name  <- paste0(plot_noexist, '_','var_','table')
            output[[selected_var_UI_name]] <- renderUI({
              NULL
            })
          })
        }
      }
    })
  # unselect all
  observeEvent({
    input$unselect_all
    -99999}, {
      req(!is.null(input$tab_1))
      predefined_plot_type <- c('figIa1', 'figIa2', 'figII', 'figIII',
                                'figIVa', 'figIVb', 'figV', 'figVI')

      plot_type_list_table <- func_pathinfo(input$exp_1, input$tab_1, 'plot_type')
      var_list_table <- func_pathinfo(input$exp_1, input$tab_1, 'var')
      
      for (plot_type_name in unique(plot_type_list_table)) { # figV figVI
        local({
          selected_var_UI_name <- ns(paste0(plot_type_name, '_','var_', 'table'))
          var <- unique(var_list_table[which(plot_type_list_table == plot_type_name)])
          checkbox <- func_plottype2(plot_type_full_list = plot_type_list_table, 
                                     var_full_list = var_list_table, plot_type_name = plot_type_name,
                                     selected_var_UI_name = selected_var_UI_name, pB_id = values$pB_id, 
                                     select_all_i = NULL, select_module = TRUE)
          selected_var_UI_name <- paste0(plot_type_name, '_','var_', 'table')
          output[[selected_var_UI_name]] <- renderUI(checkbox[[1]]) 
          values$pB_id <- checkbox[[2]]
        })
      }
    })
  
  # select all
  observeEvent({
    input$select_all
    -99999}, {
      req(!is.null(input$tab_1))
      predefined_plot_type <- c('figIa1', 'figIa2', 'figII', 'figIII',
                                'figIVa', 'figIVb', 'figV', 'figVI')
      plot_type_list_table <- func_pathinfo(input$exp_1, input$tab_1, 'plot_type')
      var_list_table <- func_pathinfo(input$exp_1, input$tab_1, 'var')
      
      for (plot_type_name in unique(plot_type_list_table)) { # figV figVI
        local({
          selected_var_UI_name <- ns(paste0(plot_type_name, '_','var_', 'table'))
          var <- unique(var_list_table[which(plot_type_list_table == plot_type_name)])
          checkbox <- func_plottype2(plot_type_full_list = plot_type_list_table, 
                                     var_full_list = var_list_table, plot_type_name = plot_type_name,
                                     selected_var_UI_name = selected_var_UI_name, pB_id = values$pB_id, 
                                     select_all_i = var, select_module = TRUE)
          
          selected_var_UI_name <- paste0(plot_type_name, '_','var_', 'table')
          output[[selected_var_UI_name]] <- renderUI(checkbox[[1]]) 
          values$pB_id <- checkbox[[2]]
        })
      }
    })
  
  var_selected <- reactive({
    var_selected_table <- c(input$figIa1_var_table, input$figIa2_var_table, 
                            input$figII_var_table, input$figIII_var_table,
                            input$figIVa_var_table, input$figIVb_var_table, 
                            input$figV_var_table, input$figVI_var_table)
    all_selected <- c(var_selected_table)
    return(all_selected)
  })
  
  plot_type_for_each_var <- reactive({
    plot_type_selected_table <- c(rep('timeseries_figIa1', length(input$figIa1_var_table)),
                                  rep('timeseries_figIa2', length(input$figIa2_var_table)),
                                  rep('timeseries_figII', length(input$figII_var_table)),
                                  rep('timeseries_figIII', length(input$figIII_var_table)),
                                  rep('timeseries_figIVa', length(input$figIVa_var_table)),
                                  rep('timeseries_figIVb', length(input$figIVb_var_table)),
                                  rep('map_figV', length(input$figV_var_table)),
                                  rep('map_figVI', length(input$figVI_var_table)))
    all_selected <- c(plot_type_selected_table)
    return(all_selected)
  })
  
  table_for_each_var <- reactive({
    table_selected_table <- rep(input$tab_1,
                                length(input$figIa1_var_table) + length(input$figIa2_var_table) + length(input$figII_var_table) +
                                length(input$figIII_var_table) + length(input$figIVa_var_table) + length(input$figIVb_var_table) +
                                length(input$figV_var_table) + length(input$figVI_var_table))
    all_selected <- c(table_selected_table)
    return(all_selected)
  })

  file_path_list <- reactive({
    req(table_for_each_var())
    req(plot_type_for_each_var())
    req(var_selected())
    
    file_path_list <- vector('list', length = length(var_selected()))
    for (kk in 1:length(var_selected())) {
      path <- file.path('/esarchive/exp/ecearth', input$exp_1, 'plots_s2d',
                        table_for_each_var()[kk], var_selected()[kk])
      file_path_list[[kk]] <- Sys.glob(paste0(path, '/', plot_type_for_each_var()[kk], '_*'))
    }
    if (!identical(file_path_list, character(0))) {
      sort_file_path_list <- file_path_list
    }
    return(sort_file_path_list)
  })

  observe({
    file_path_vals <- file_path_list()    
    file_path_vals <- file_path_vals[lengths(file_path_vals) != 0]
  })

  observeEvent(input$nextPlot, {
    req(!is.null(input$exp_1))
    req(!is.null(var_selected()))
    req(!is.null(table_for_each_var()))
    req(!is.null(file_path_list()))
    req(!identical(character(0), file_path_list()))

    func_name <- function(x) {
      for (kk in 1:length(x)) {
        for (jj in 1:length(x[[kk]])) {
          x[[kk]][jj] <- paste0(names(x)[kk], '_', jj)
        }
      }
      return(x)
    }
   
    file_path_list <- isolate(file_path_list())
    file_path_list <- file_path_list[lengths(file_path_list) != 0]
    file_path_lista <- file_path_list
    
    output$plots <- renderUI({
      plot_output_list <- lapply(1:length(file_path_lista), function(i) {
        ns <- session$ns
        plotname <- ns(paste("plot", i, sep=""))
        plotOutput(plotname, width = figure_size)
      })
      do.call(tagList, plot_output_list)
    })
    observe({
      for (i in 1:length(file_path_lista)) {
        local({
          my_i <- i
          plotname <- paste("plot", my_i, sep="")
          output[[plotname]] <- renderImage({
            list(src = normalizePath(paste0(file_path_lista[[my_i]][1])),width = figure_size)
          }, deleteFile = FALSE)
        })
      }
    }) 
  })

  observeEvent({
    req(is.null(var_selected()))
    input$unselect_all
    input$nextPlot}, {
      output$plots <- renderText('Nothing selected')
    })
}

# -------------------------------- main code ui --------------------------------

ui <- fluidPage(
  shinyjs::useShinyjs(),
  # theme = bs_theme(version = 4, bootswatch = "cosmo"),
  titlePanel("Experiments visualization tool"),
  navbarPage('Pages',
  tabPanel('Same experiment',
  
  titlePanel("ECEARTH experiment diagnostics/cmorfiles visualization"),
  h5(paste0("Select expID/member/variables, click 'Show figures', and the figures will be printed below.")),
  h6(tags$em('Note1: The images could be zoomed in by "right click -> open in a new window".', 
	       style="color:Crimson; font-weight: bold;")),
  tags$style("
    .checkbox { /* checkbox is a div class*/
      line-height: auto;
      margin-bottom: auto; /*set the margin, so boxes don't overlap*/
      margin-top: 5px;
      font-size: 12px;
    }
    input[type='checkbox'] { /* style for checkboxes */
      width: 10px; /*Desired width*/
      height: 10px; /*Desired height*/
      line-height: auto;
    }
    span {
      margin-left: 0px;  /*set the margin, so boxes don't overlap labels*/
      line-height: 100%;
    }
  "),

  div(style="overflow-x: auto;",
    fluidRow( 
        class = "flex-nowrap",
        tags$head(tags$style("
        .flex-nowrap {
          display: inline-flex;
          -webkit-flex-wrap: nowrap !important;
          -ms-flex-wrap: nowrap !important;
          flex-wrap: nowrap !important;
          flex-direction: row;
          height:500px;
          !width:100%;
          overflow-y: auto;
          overflow-x: hidden;
        }
        .custom-column {
          width: 25vw;
          margin: 0px 2px;
        }
        .custom-column2 {
          width: 150px;
          margin: 0px 5px;
        }
        ")),

    div(class = "custom-column2",
              wellPanel(
                  uiOutput("exp"),
                  actionButton("unselect_all", "Unselect all", style = "padding: 8px 21px 8px 21px; font-size:90%"),
      tags$br(),
                  tags$br(),
      actionButton("nextPlot","Show figures",
                                style = "color: #fef9e7; background-color: #f1c40f; border: none;
                                padding: 8px 18px 8px 18px; font-size:90%"),
              h6(tags$em('Finish choosing? Click here ↑',
                      style="color:DarkOrange; font-weight: bold;")),
                  tags$br(),
                  sliderInput(inputId = 'metrics_size', label = 'Number of figures per line',
                              min = 1, max = 4, step = 1, value = 2,
                              round = TRUE, animate = FALSE, ticks = TRUE)
              )),
    div(id = 'div_ocean', class = "custom-column",
              wellPanel(
                  fluidRow(
                      column(6, 
                              h5(tags$b(style="background-color:MidnightBlue;color:SeaShell", 
                                        tags$b("[Ocean]")))),
                      column(6, 
            div(class = "select_all_btn",
                              actionButton("select_ocean", "Select all" ,
                                          style = "font-size: 8px; 
            border-radius: 10px; outline: none; width: 70px;
                                    color: black; background-color: white; border-color: green;
                              padding: 2px 2px; position: relative; left: 15px; bottom: 0px;"
            )))
                  ),
                  
                  div(id = 'plottype_global_ocean',
                      h5(tags$b(style="background-color:LightBlue", tags$b("Types of figures")))),
                              uiOutput("figIa1_var_ocean"),
                              uiOutput("figIa2_var_ocean"),
                              uiOutput("figII_var_ocean"),
                              uiOutput("figIII_var_ocean"),
                              uiOutput("figIVa_var_ocean"),
                              uiOutput("figIVb_var_ocean"),
                              uiOutput("figV_var_ocean"),
                              uiOutput("figVI_var_ocean"),
                  hr()
          )),
    div(id = 'div_atmos', class = "custom-column", 
              wellPanel(
                  fluidRow(
                      column(6,
                              h5(tags$b(style="background-color:MidnightBlue;color:SeaShell",
                                        tags$b("[Atmos]")))),
                      column(6,
                              actionButton("select_atmos", "Select all",
                                          style="font-size: 8px; border-radius: 10px; outline:none; width: 70px;
                      color: black; background-color: white; border-color: green;
                      padding: 2px 2px; position: relative; left: 15px; bottom: 0px;"))
                  ),
                  
                  div(id = 'plottype_global_clim_atmos',
                      h5(tags$b(style="background-color:LightBlue", tags$b("Types of figures")))),
                              uiOutput("figIa1_var_atmos"),
                              uiOutput("figIa2_var_atmos"),
                              uiOutput("figII_var_atmos"),
                              uiOutput("figIII_var_atmos"),
                              uiOutput("figIVa_var_atmos"),
                              uiOutput("figIVb_var_atmos"),
                              uiOutput("figV_var_atmos"),
                              uiOutput("figVI_var_atmos"),
                  hr()
              )),
    div(id = 'div_seaice', class = "custom-column",
              wellPanel(
                  fluidRow(
                      column(6, 
                              h5(tags$b(style="background-color:MidnightBlue;color:SeaShell", 
                                        tags$b("[Sea Ice]")))),
                      column(6, 
            div(class = "select_all_btn",
                              actionButton("select_seaice", "Select all" ,
                                          style = "font-size: 8px; 
            border-radius: 10px; outline: none; width: 70px;
                                    color: black; background-color: white; border-color: green;
                              padding: 2px 2px; position: relative; left: 15px; bottom: 0px;"
            )))
                  ),
                  
                  div(id = 'plottype_global_seaice',
                      h5(tags$b(style="background-color:LightBlue", tags$b("Types of figures")))),
                              uiOutput("figIa1_var_seaice"),
                              uiOutput("figIa2_var_seaice"),
                              uiOutput("figII_var_seaice"),
                              uiOutput("figIII_var_seaice"),
                              uiOutput("figIVa_var_seaice"),
                              uiOutput("figIVb_var_seaice"),
                              uiOutput("figV_var_seaice"),
                              uiOutput("figVI_var_seaice"),
                  hr()    
          ))


    )
  ), # end div columns

  includeScript('shiny.js'),
  hr(),
  uiOutput("plots"),
  tags$head(tags$style("#plots{color:MidnightBlue;
                                 font-size:25px;
                                 }"
    )
    ),
  tags$script('
    document.getElementById("exp").onclick = function() {

      Shiny.onInputChange("figIa1_var_ocean", null);
      Shiny.onInputChange("figIa2_var_ocean", null);
      Shiny.onInputChange("figII_var_ocean", null);
      Shiny.onInputChange("figIII_var_ocean", null);
      Shiny.onInputChange("figIVa_var_ocean", null);
      Shiny.onInputChange("figIVb_var_ocean", null);
      Shiny.onInputChange("figV_var_ocean", null);
      Shiny.onInputChange("figVI_var_ocean", null);

      Shiny.onInputChange("figIa1_var_atmos", null);
      Shiny.onInputChange("figIa2_var_atmos", null);
      Shiny.onInputChange("figII_var_atmos", null);
      Shiny.onInputChange("figIII_var_atmos", null);
      Shiny.onInputChange("figIVa_var_atmos", null);
      Shiny.onInputChange("figIVb_var_atmos", null);
      Shiny.onInputChange("figV_var_atmos", null);
      Shiny.onInputChange("figVI_var_atmos", null);

      Shiny.onInputChange("figIa1_var_seaice", null);
      Shiny.onInputChange("figIa2_var_seaice", null);
      Shiny.onInputChange("figII_var_seaice", null);
      Shiny.onInputChange("figIII_var_seaice", null);
      Shiny.onInputChange("figIVa_var_seaice", null);
      Shiny.onInputChange("figIVb_var_seaice", null);
      Shiny.onInputChange("figV_var_seaice", null);
      Shiny.onInputChange("figVI_var_seaice", null);

    };
  '),

  tags$script(HTML('
                    mybutton = document.getElementById("goupbtn");

                    // When the user scrolls down 20px from the top of the document, show the button
                    window.onscroll = function() {scrollFunction()};

                    function scrollFunction() {
                      if (document.body.scrollTop > 20 || document.documentElement.scrollTop > 20) {
                        mybutton.style.display = "block";
                      } else {
                        mybutton.style.display = "none";
                      }
                    }

                    // When the user clicks on the button, scroll to the top of the document
                    function topFunction() {
                      document.body.scrollTop = 0; // For Safari
                      document.documentElement.scrollTop = 0; // For Chrome, Firefox, IE and Opera
                    }
  ')),
    
    #NOTE: the 'id="goupbtn" is useless. Need to find a way to make the above script work.
    HTML('<button onclick="topFunction()"; /!id="goupbtn" 
		      style="position: fixed; bottom: 20px; right: 30px;
                             z-index: 99; border: none; outline: none;
                             background-color: Gold; opacity:0.6; 
			     color: SaddleBrown; cursor: pointer; 
                             padding: 8px; border-radius: 8px; 
			     font-size: 12px; font-weight: bold;"
                      >Top'),
#		      title="Go to top">Top</button>')),
    
    #popup box
    ##The "Modal" part:
    div(
        id = 'busyModal', class = 'modal', role = 'dialog', 'data-backdrop' = 'static',
        div(
            class = 'modal-dialog modal-sm',
            
            ##The "Modal content" part:
            div(
                class = 'modal-content',
                div(class = 'modal-header', 
                    h5(class = 'modal-title', 'Connecting to the server...')),
                div(class = 'modal-body', 
                    p(paste('This dialog box will disappear',
                            'automatically after shiny is ready.')))
            )
        )
    )

# modules inside ui

  ),
  navbarMenu("Different experiments",
  tabPanel('1 experiment', 
  fluidPage(
    fluidRow(
      div(id = 'div_exp1',
          fluidRow(
            column(12, wellPanel(
              MultipleExpUI('1')
            ))
          )
        )
    )
  )

  ), # end tabpanel 1
  tabPanel('2 experiments', 
  fluidPage(
    fluidRow(
      div(id = 'div_exp2',
          fluidRow(
            column(6, wellPanel(
              MultipleExpUI('2')
            )),
            column(6, wellPanel(
              MultipleExpUI('3')
            ))
          )
        )
    )
  )

  ), # end tabpanel 1
  tabPanel('3 experiments', 
  fluidPage(
    fluidRow(
      div(id = 'div_exp3',
          fluidRow(
            column(4, wellPanel(
              MultipleExpUI('4')
            )),
            column(4, wellPanel(
              MultipleExpUI('5')
            )),
            column(4, wellPanel(
              MultipleExpUI('6')
            ))
          )
        )
    )
  )

  ), # end tabpanel 2, # end tabpanel 1
  tabPanel('4 experiments', 
  fluidPage(
    fluidRow(
      div(id = 'div_exp4',
          fluidRow(
            column(3, wellPanel(
              MultipleExpUI('7')
            )),
            column(3, wellPanel(
              MultipleExpUI('8')
            )),
            column(3, wellPanel(
              MultipleExpUI('9')
            )),
            column(3, wellPanel(
              MultipleExpUI('10')
            ))
          )
        )
    )
  )

  ) # end tabpanel 2
  ) # end navbarMenu
  ))

# ------------------------------ main code server ------------------------------

server <- function(input, output, session) {
  observe({
    # check directory /plots_s2d
    asd <- list.files(file.path('/esarchive/exp/ecearth/'))
    asd_fullpath <- lapply(asd, function(x) paste0('/esarchive/exp/ecearth/', x, '/'))
    qwe <- lapply(asd_fullpath, function(x) list.files(x))
    qwe <- sapply(qwe, function(x) {if (!('plots_s2d' %in% x)) {x <- 0} else {x <- 1} })
    asd <- asd[which(qwe == 1)]

    # check tables
    fullpathexp <- lapply(asd, function(x) paste0('/esarchive/exp/ecearth/', x, '/plots_s2d/'))
    qwexp <- lapply(fullpathexp, function(x) list.files(x))
    qwexp <- sapply(qwexp, function(x) {if (!(all(x %in% c('atmos', 'ocean', 'seaice')))) {x <- 0} else {x <- 1} })
    asd <- asd[which(qwexp == 1)]

    output$exp <- renderUI({
      selectizeInput("exp", "Experiment ID:", choices = asd, selected = "a3w5")
    }) #renderUI, output$exp_1

  })
  exp_1r <- reactive(input$exp)

  tables <- eventReactive({
    input$exp
    1}, {
      req(!is.null(input$exp))
      opt_table <- list.files(file.path('/esarchive/exp/ecearth',
                                              input$exp, 'plots_s2d'))
      return(opt_table)
    })

  observeEvent({
    input$exp
    tables()
    -99999}, {
      req(!is.null(tables()) & !identical(tables(), character(0)))
      predefined_plot_type <- c('figIa1', 'figIa2', 'figII', 'figIII',
                                'figIVa', 'figIVb', 'figV', 'figVI')
      cat_global <- c('figIa1', 'figIa2', 'figII', 'figIII',
                      'figIVa', 'figIVb', 'figV', 'figVI')

      select_values_table_i <- c(values$select_ocean_i, values$select_atmos_i, values$select_seaice_i)

      for (i in 1:length(tables())) {

        plot_type_list_table <- func_pathinfo(input$exp, tables()[i], 'plot_type')
        var_list_table <- func_pathinfo(input$exp, tables()[i], 'var')

        for (plot_type_name in unique(plot_type_list_table)) {
          local({
            selected_var_UI_name <- paste0(plot_type_name, '_','var_', tables()[i])
            checkbox <- func_plottype2(plot_type_list_table, var_list_table, plot_type_name, 
                                      selected_var_UI_name, values$pB_id, select_values_table_i[i])
            output[[selected_var_UI_name]] <- renderUI(checkbox[[1]])
            values$pB_id <- checkbox[[2]]
          })
        }

        if (!all(predefined_plot_type %in% plot_type_list_table)) {
          plot_type_name_noexist <- predefined_plot_type[which(!(predefined_plot_type %in% plot_type_list_table))]
          for (plot_noexist in plot_type_name_noexist) {
              local({
                  selected_var_UI_name  <- paste0(plot_noexist, '_','var_',tables()[i])
                  output[[selected_var_UI_name]] <- renderUI({
                      NULL
                  })
              }) #local
          } #for
          id_table <- paste0('plottype_global_', tables()[i])
          if (all(cat_global %in% plot_type_name_noexist)) {
              shinyjs::hide(id = id_table)
          } else {
              shinyjs::show(id = id_table)
          }
        }
      }
      # ------------------ select/deselect all
      # ocean
      observeEvent({
        values$select_ocean_i
        input$unselect_all
        -99999}, {
          req('ocean' %in% tables())

          plot_type_list_ocean <- func_pathinfo(input$exp, 'ocean', 'plot_type')
          var_list_ocean <- func_pathinfo(input$exp, 'ocean', 'var')
          
          plot_types <- paste0(unique(plot_type_list_ocean), '_ocean')
          plot_type_output_list <- lapply(1:length(plot_types),
                                          function(i) {uiOutput(plot_types[i])})

          for (nn in 1:length(unique(plot_type_list_ocean))) {
            local({
                my_nn <- nn
                plot_type_name <- unique(plot_type_list_ocean)[my_nn]
                if (plot_type_name %in% predefined_plot_type) {
                    selected_var_UI_name <- paste0(plot_type_name, '_var_', 'ocean')
                } else {
                    selected_var_UI_name <- paste0(plot_type_name, '_ocean')
                }
                
                ind <- which(plot_type_list_ocean == plot_type_name)
                var <- var_list_ocean[ind]
                if (values$select_ocean_i %% 2 == 0) {
                    updateActionButton(session, 'select_ocean', label = 'Select all') 
                    checkbox <- func_plottype2(plot_type_list_ocean, var_list_ocean, plot_type_name, 
                                              selected_var_UI_name, values$pB_id, values$select_ocean_i)
                    output[[selected_var_UI_name]] <- renderUI(checkbox[[1]])
                    values$pB_id <- checkbox[[2]]
                } else {
                    updateActionButton(session, 'select_ocean', label = 'Deselect all')
                    checkbox <- func_plottype2(plot_type_list_ocean, var_list_ocean, plot_type_name, 
                                              selected_var_UI_name, values$pB_id, values$select_ocean_i)
                    
                    output[[selected_var_UI_name]] <- renderUI(checkbox[[1]])
                    values$pB_id <- checkbox[[2]]
                }
            })
        }  #for loop
      })
      # atmos
      observeEvent({
          values$select_atmos_i
          input$unselect_all
          -99999}, {
              req('atmos' %in% tables())
              
              plot_type_list_atmos <- func_pathinfo(input$exp, 'atmos', 'plot_type')
              var_list_atmos <- func_pathinfo(input$exp, 'atmos', 'var')
              
              plot_types <- paste0(unique(plot_type_list_atmos), '_atmos')
              plot_type_output_list <- lapply(1:length(plot_types),
                                              function(i) {uiOutput(plot_types[i])})
              
              for (nn in 1:length(unique(plot_type_list_atmos))) {
                  local({
                      my_nn <- nn
                      plot_type_name <- unique(plot_type_list_atmos)[my_nn]
                      if (plot_type_name %in% predefined_plot_type) {
                          selected_var_UI_name <- paste0(plot_type_name, '_var_', 'atmos')
                      } else {
                          selected_var_UI_name <- paste0(plot_type_name, '_atmos')
                      }
                      
                      ind <- which(plot_type_list_atmos == plot_type_name)
                      var <- var_list_atmos[ind]
                      
                      if (values$select_atmos_i %% 2 == 0) {
                          updateActionButton(session, 'select_atmos', label = 'Select all')
                          # generate the UI
                          checkbox <- func_plottype2(plot_type_list_atmos, var_list_atmos, plot_type_name,
                                                    selected_var_UI_name, values$pB_id, values$select_atmos_i)
                          
                          output[[selected_var_UI_name]] <- renderUI(checkbox[[1]])
                          values$pB_id <- checkbox[[2]]
                          
                      } else {
                          updateActionButton(session, 'select_atmos', label = 'Deselect all')
                          # generate the UI
                          checkbox <- func_plottype2(plot_type_list_atmos, var_list_atmos, plot_type_name,
                                                    selected_var_UI_name, values$pB_id, values$select_atmos_i)
                          
                          output[[selected_var_UI_name]] <- renderUI(checkbox[[1]])
                          values$pB_id <- checkbox[[2]]
                      }
                  })
              }  #for loop
          })
      # seaice
      observeEvent({
          values$select_seaice_i
          input$unselect_all
          -99999}, {
              req('seaice' %in% tables())
              
              plot_type_list_seaice <- func_pathinfo(input$exp, 'seaice', 'plot_type')
              var_list_seaice <- func_pathinfo(input$exp, 'seaice', 'var')
              
              plot_types <- paste0(unique(plot_type_list_seaice), '_seaice')
              plot_type_output_list <- lapply(1:length(plot_types),
                                              function(i) {uiOutput(plot_types[i])})
              
              for (nn in 1:length(unique(plot_type_list_seaice))) {
                  local({
                      my_nn <- nn
                      plot_type_name <- unique(plot_type_list_seaice)[my_nn]
                      if (plot_type_name %in% predefined_plot_type) {
                          selected_var_UI_name <- paste0(plot_type_name, '_var_', 'seaice')
                      } else {
                          selected_var_UI_name <- paste0(plot_type_name, '_seaice')
                      }
                      
                      ind <- which(plot_type_list_seaice == plot_type_name)
                      var <- var_list_seaice[ind]
                      
                      if (values$select_seaice_i %% 2 == 0) {
                          updateActionButton(session, 'select_seaice', label = 'Select all')
                          # generate the UI
                          checkbox <- func_plottype2(plot_type_list_seaice, var_list_seaice, plot_type_name,
                                                    selected_var_UI_name, values$pB_id, values$select_seaice_i)
                          
                          output[[selected_var_UI_name]] <- renderUI(
                              checkbox[[1]]
                          )
                          values$pB_id <- checkbox[[2]]
                          
                      } else {
                          updateActionButton(session, 'select_seaice', label = 'Deselect all')
                          # generate the UI
                          checkbox <- func_plottype2(plot_type_list_seaice, var_list_seaice, plot_type_name,
                                                    selected_var_UI_name, values$pB_id, values$select_seaice_i)
                          
                          output[[selected_var_UI_name]] <- renderUI(checkbox[[1]])
                          values$pB_id <- checkbox[[2]]
                      }
                  })
              }  #for loop
          })

      # Hide the div box if there is no this CMOR table
      if (!('ocean' %in% tables())) {
        shinyjs::hide(id = "div_ocean")
      } else  {
        shinyjs::show(id = "div_ocean")
      }

      if (!('atmos' %in% tables())) {
          shinyjs::hide(id = "div_atmos")
      } else {
          shinyjs::show(id = "div_atmos")
      }

      if (!('seaice' %in% tables())) {
          shinyjs::hide(id = "div_seaice")
      } else {
          shinyjs::show(id = "div_seaice")
      }
  })

  observe({
    shinyjs::toggleState("nextPlot", !is.null(var_selected()) && var_selected() != "")
  })

  var_selected <- reactive({
    req(!is.null(input$exp))
    var_selected_ocean <- c(input$figIa1_var_ocean, input$figIa2_var_ocean,
                            input$figII_var_ocean, input$figIII_var_ocean,
                            input$figIVa_var_ocean, input$figIVb_var_ocean,
                            input$figV_var_ocean, input$figVI_var_ocean)

    var_selected_atmos <- c(input$figIa1_var_atmos, input$figIa2_var_atmos,
                            input$figII_var_atmos, input$figIII_var_atmos,
                            input$figIVa_var_atmos, input$figIVb_var_atmos,
                            input$figV_var_atmos, input$figVI_var_atmos)

    var_selected_seaice <- c(input$figIa1_var_seaice, input$figIa2_var_seaice,
                            input$figII_var_seaice, input$figIII_var_seaice,
                            input$figIVa_var_seaice, input$figIVb_var_seaice,
                            input$figV_var_seaice, input$figVI_var_seaice)
    
    all_selected <- c(var_selected_ocean, var_selected_atmos, var_selected_seaice)

    return(all_selected)

  })

  plot_type_for_each_var <- reactive({

    #--------- ocean -----------
    plot_type_list <- func_pathinfo(input$exp, 'ocean', 'plot_type')

    plot_type_selected_ocean <- c(rep('timeseries_figIa1', length(input$figIa1_var_ocean)),
                                  rep('timeseries_figIa2', length(input$figIa2_var_ocean)),
                                  rep('timeseries_figII', length(input$figII_var_ocean)),
                                  rep('timeseries_figIII', length(input$figIII_var_ocean)),
                                  rep('timeseries_figIVa', length(input$figIVa_var_ocean)),
                                  rep('timeseries_figIVb', length(input$figIVb_var_ocean)),
                                  rep('map_figV', length(input$figV_var_ocean)),
                                  rep('map_figVI', length(input$figVI_var_ocean)))
    #--------- atmos -----------
    plot_type_list <- func_pathinfo(input$exp, 'atmos', 'plot_type')

    plot_type_selected_atmos <- c(rep('timeseries_figIa1', length(input$figIa1_var_atmos)),
                                  rep('timeseries_figIa2', length(input$figIa2_var_atmos)),
                                  rep('timeseries_figII', length(input$figII_var_atmos)),
                                  rep('timeseries_figIII', length(input$figIII_var_atmos)),
                                  rep('timeseries_figIVa', length(input$figIVa_var_atmos)),
                                  rep('timeseries_figIVb', length(input$figIVb_var_atmos)),
                                  rep('map_figV', length(input$figV_var_atmos)),
                                  rep('map_figVI', length(input$figVI_var_atmos)))
    #--------- seaice -----------
    plot_type_list <- func_pathinfo(input$exp, 'seaice', 'plot_type')

    plot_type_selected_seaice <- c(rep('timeseries_figIa1', length(input$figIa1_var_seaice)),
                                  rep('timeseries_figIa2', length(input$figIa2_var_seaice)),
                                  rep('timeseries_figII', length(input$figII_var_seaice)),
                                  rep('timeseries_figIII', length(input$figIII_var_seaice)),
                                  rep('timeseries_figIVa', length(input$figIVa_var_seaice)),
                                  rep('timeseries_figIVb', length(input$figIVb_var_seaice)),
                                  rep('map_figV', length(input$figV_var_seaice)),
                                  rep('map_figVI', length(input$figVI_var_seaice)))
    # all

    all_selected <- c(plot_type_selected_ocean, plot_type_selected_atmos, plot_type_selected_seaice)

    return(all_selected)
  })

  table_for_each_var <- reactive({

    #--------- ocean -----------
    table_selected_ocean <- rep('ocean',
                                length(input$figIa1_var_ocean) + length(input$figIa2_var_ocean) + length(input$figII_var_ocean) +
                                length(input$figIII_var_ocean) + length(input$figIVa_var_ocean) + length(input$figIVb_var_ocean) +
                                length(input$figV_var_ocean) + length(input$figVI_var_ocean))
    #--------- atmos -----------
    table_selected_atmos <- rep('atmos',
                                length(input$figIa1_var_atmos) + length(input$figIa2_var_atmos) + length(input$figII_var_atmos) +
                                length(input$figIII_var_atmos) + length(input$figIVa_var_atmos) + length(input$figIVb_var_atmos) +
                                length(input$figV_var_atmos) + length(input$figVI_var_atmos))
    #--------- seaice -----------
    table_selected_seaice <- rep('seaice',
                                length(input$figIa1_var_seaice) + length(input$figIa2_var_seaice) + length(input$figII_var_seaice) +
                                length(input$figIII_var_seaice) + length(input$figIVa_var_seaice) + length(input$figIVb_var_seaice) +
                                length(input$figV_var_seaice) + length(input$figVI_var_seaice))
    # all

    all_selected <- c(table_selected_ocean, table_selected_atmos, table_selected_seaice)

    return(all_selected)
  })
  
  file_path_list <- reactive({
    req(table_for_each_var())
    req(plot_type_for_each_var())
    req(var_selected())

    file_path_list <- vector('list', length = length(var_selected()))
    for (kk in 1:length(var_selected())) {
      path <- file.path('/esarchive/exp/ecearth', input$exp, 'plots_s2d',
                        table_for_each_var()[kk], var_selected()[kk])
      file_path_list[[kk]] <- Sys.glob(paste0(path, '/', plot_type_for_each_var()[kk], '_*'))
      
    }
    sort_file_path_list <- vector('list', length = length(unique(plot_type_for_each_var())))
    names(sort_file_path_list) <- unique(plot_type_for_each_var())

    files_only <- lapply(lapply(lapply(file_path_list, strsplit, split = '/', fixed = TRUE), 
                                    '[[', 1),'[', 9)
    plottype_only <- paste0(lapply(lapply(lapply(files_only, strsplit, split = '_', fixed = TRUE), 
                                    '[[', 1),'[', 1),'_',lapply(lapply(lapply(files_only, strsplit, split = '_', fixed = TRUE), 
                                    '[[', 1),'[', 2))
    for (kk in 1:length(sort_file_path_list)) {
        logi <- names(sort_file_path_list)[kk] == plottype_only
        sort_file_path_list[[kk]] <- file_path_list[which(logi)]
    }

    return(sort_file_path_list)

  })
  

#--------------reactiveValues--------------
values <- reactiveValues()
if (!exists('values$pB_id')) values$pB_id <- 1
if (!exists('values$select_ocean_i')) values$select_ocean_i <- 0
if (!exists('values$select_atmos_i')) values$select_atmos_i <- 0
if (!exists('values$select_seaice_i')) values$select_seaice_i <- 0

observeEvent({
    input$unselect_all
    input$exp
    1}, {
        values$select_ocean_i <- 0
        values$select_atmos_i <- 0
        values$select_seaice_i <- 0

})

observeEvent({
    input$select_ocean
    1}, {
        values$select_ocean_i <- values$select_ocean_i + 1
}, priority = 1)
observeEvent({
    input$select_atmos
    1}, {
        values$select_atmos_i <- values$select_atmos_i + 1
    }, priority = 1)
observeEvent({
    input$select_seaice
    1}, {
        values$select_seaice_i <- values$select_seaice_i + 1
    }, priority = 1)

#---------------Print figures--------------------
observeEvent(input$nextPlot, {
  req(!is.null(input$exp))
  req(!is.null(var_selected()))
  req(!is.null(table_for_each_var()))
  req(!is.null(file_path_list()))

    func_name <- function(x) {
        for (kk in 1:length(x)) {
            for (jj in 1:length(x[[kk]])) {
                x[[kk]][jj] <- paste0(names(x)[kk], '_', jj)
            }
        }
        return(x)
    }
    file_path_list <- isolate(file_path_list())
    output$plots <- renderUI({

                plotoutput_list <- func_name(file_path_list)
                figure_size <- (100/input$metrics_size) - 1
                plotoutput_list <- unlist(plotoutput_list)
                figure_list <- vector('list', length(plotoutput_list))
            
                for (jj in 1:length(figure_list)) {
                    figure_list[[jj]] <- plotOutput(plotoutput_list[jj],
                                                    width = paste0((100/input$metrics_size) - 1, '%'),
                                                    height = paste0((100/input$metrics_size) - 1, '%'),
                                                    inline = T
                    )
                }
                tagList(figure_list)
            })
    #------Call renderPlot for each one-----------

    for (kk in 1:length(file_path_list)) { # plot_type

      for (nn in 1:length(file_path_list[[kk]])) {  # file
            
            local({
                my_kk <- kk
                my_nn <- nn
                plotname <- paste0(names(file_path_list)[my_kk], '_', my_nn)

                output[[plotname]] <- renderImage({
                    list(src = file_path_list[[my_kk]][[my_nn]][1], 
                        width = paste0((100/input$metrics_size) - 1, '%'), 
                        height = paste0((100/input$metrics_size) - 1, '%'))
                }, deleteFile = FALSE)
            })
            
        } #for loop nn
    } #for loop kk
})


#---------------------------------------
rv <- reactiveValues(pB_id = 1)

callModule(MultipleExpServer, '1', figure_size = "100%", reactive(rv$pB_id))
rv <- reactiveValues(pB_id = 1)
callModule(MultipleExpServer, '2', figure_size = "100%", reactive(rv$pB_id))
rv <- reactiveValues(pB_id = 1)
callModule(MultipleExpServer, '3', figure_size = "100%", reactive(rv$pB_id))
rv <- reactiveValues(pB_id = 1)
callModule(MultipleExpServer, '4', figure_size = "100%", reactive(rv$pB_id))
rv <- reactiveValues(pB_id = 1)
callModule(MultipleExpServer, '5', figure_size = "100%", reactive(rv$pB_id))
rv <- reactiveValues(pB_id = 1)
callModule(MultipleExpServer, '6', figure_size = "100%", reactive(rv$pB_id))
rv <- reactiveValues(pB_id = 1)
callModule(MultipleExpServer, '7', figure_size = "100%", reactive(rv$pB_id))
rv <- reactiveValues(pB_id = 1)
callModule(MultipleExpServer, '8', figure_size = "100%", reactive(rv$pB_id))
rv <- reactiveValues(pB_id = 1)
callModule(MultipleExpServer, '9', figure_size = "100%", reactive(rv$pB_id))
rv <- reactiveValues(pB_id = 1)
callModule(MultipleExpServer, '10', figure_size = "100%", reactive(rv$pB_id))


}

# run app
shinyApp(ui = ui, server = server)
